@@ -1,6 +1,6 @@
 ;;; essl-sas.el --- SAS customization
 
-;; Copyright (C) 1997--2001 Richard M. Heiberger, A. J. Rossini, 
+;; Copyright (C) 1997--2004 Richard M. Heiberger, A. J. Rossini, 
 ;;  M. Maechler, Rodney Sparapani
 
 ;; Authors: Richard M. Heiberger <rmh@astro.ocis.temple.edu>,
@@ -9,9 +9,9 @@
 ;; Maintainer: Richard M. Heiberger <rmh@astro.ocis.temple.edu>,
 ;;             Rodney Sparapani <rsparap@mcw.edu>
 ;; Created: 20 Aug 1997
-;; Modified: $Date: 2002/05/24 23:27:41 $
-;; Version: $Revision: 1.1 $
-;; RCS: $Id: essl-sas.el,v 1.1 2002/05/24 23:27:41 rossini Exp $
+;; Modified: $Date: 2004/07/02 14:46:35 $
+;; Version: $Revision: 1.3 $
+;; RCS: $Id: essl-sas.el,v 1.3 2004/07/02 14:46:35 rsparapa Exp $
 ;;
 ;; Keywords: start up, configuration.
 
@@ -113,8 +113,10 @@ the mode line."
   (use-local-map sas-mode-local-map)
   (toggle-read-only t)) ;; to protect the buffer.
 
-(setq auto-mode-alist (append '(("\\.lst\\'" . sas-listing-mode)) auto-mode-alist))
-(setq auto-mode-alist (append '(("\\.LST\\'" . sas-listing-mode)) auto-mode-alist))
+(if (not (featurep 'xemacs)) 
+    (setq auto-mode-alist 
+	(append '(("\\.[lL][sS][tT]\\'" . sas-listing-mode)) auto-mode-alist)))
+;;;###autoload(add-to-list 'auto-mode-alist '("\\.[lL][sS][tT]\\'" . sas-listing-mode))
 
 (fset 'sas-log-mode        'SAS-log-mode)
 (fset 'SAS-transcript-mode 'SAS-log-mode)
@@ -168,7 +170,7 @@ the mode line."
 (defcustom sas-get-options nil 
   "Options to be passed to SAS in sas-get-dataset."
   :group 'ess-sas
-  :type  'string)
+  :type '(choice (const nil) string))
 
 (defcustom sas-get-options-history nil
   "History list of Options passed to SAS in sas-get-dataset."
@@ -193,7 +195,7 @@ number."
 (defcustom sas-file-name nil
   "*The name of the current sas file."
   :group 'ess-sas
-  :type  'file)
+  :type '(choice (const nil) file))
 
 ;; The next two are ``the inside of [...] in a regexp'' to be used in
 ;; (skip-chars-(for|back)ward SAS-..-chars)
@@ -210,7 +212,7 @@ number."
 (defcustom ess-sas-run-make-regexp nil
   "If you do not want to run make-regexp, then set to nil."
   :group 'ess-sas
-  :type  'string)
+  :type '(choice (const nil) string))
 
 (require 'essa-sas)
 
@@ -395,7 +397,7 @@ number."
 						    font-lock-reference-face)
 
 	 (cons (concat "\\(^[0-9]*\\|;\\|):\\|%then\\|%else\\)[ \t]*"
-		"\\(data\\|endsas\\|quit\\|run\\)[ \t\n;]")
+		"\\(data\\|endsas\\|finish\\|quit\\|run\\|start\\)[ \t\n;]")
 		      				    font-lock-reference-face)
 	 (cons (concat "\\(^[0-9]*\\|;\\|):\\|%then\\|%else\\)[ \t]*"
 		"proc[ \t]+[a-z][a-z_0-9]+")        font-lock-reference-face)
@@ -423,32 +425,40 @@ number."
 						    font-lock-keyword-face)
 
 	 ;; SAS statements
-
 	 (cons (concat
 		"\\(^[0-9]*\\|):\\|[;,]\\|then\\|else\\)[ \t]*"
 		"\\(a\\(bort\\|rray\\|ttrib\\)\\|by"
 		"\\|c\\(hange\\|lass\\|ontrast\\)"
-		"\\|d\\(elete\\|isplay\\|m\\|rop\\)"
+		"\\|d\\(elete\\|isplay\\|m\\|o\\([ \t]+\\(data\\|over\\)\\)?\\|rop\\)"
 		"\\|e\\(rror\\|stimate\\|xc\\(hange\\|lude\\)\\)"
 		"\\|f\\(ile\\(name\\)?\\|o\\(otnote\\(10?\\|[2-9]\\)?\\|rmat\\)\\|req\\)"
 		"\\|go\\([ \t]*to\\|ptions\\)"
 		"\\|i\\(d\\|f\\|n\\(dex\\|f\\(ile\\|ormat\\)\\|put\\|value\\)\\)"
-		"\\|keep\\|l\\(abel\\|ength\\|i\\(bname\\|nk\\)\\|smeans\\)"
+		"\\|keep\\|l\\(abel\\|ength\\|i\\(bname\\|nk\\|st\\)\\|smeans\\)"
 		"\\|m\\(anova\\|e\\(ans\\|rge\\)\\|issing\\|od\\(el\\|ify\\)\\)\\|note"
 		"\\|o\\(ptions\\|therwise\\|utput\\)\\|p\\(lot\\|ut\\)"
 		"\\|r\\(andom\\|e\\(name\\|peated\\|tain\\)\\)"
 		"\\|s\\(ave\\|e\\(lect\\|t\\)\\|kip\\|trata\\|umby\\)"
 		"\\|t\\(ables?\\|i\\(me\\|tle\\(10?\\|[2-9]\\)?\\)\\)\\|update"
-		"\\|va\\(lue\\|r\\)\\|w\\(eight\\|here\\|i\\(ndow\\|th\\)\\)\\)"
+		"\\|va\\(lue\\|r\\)\\|w\\(eight\\|here\\|i\\(ndow\\|th\\)\\)"
+
+	;; IML statements that are not also SAS statements
+		"\\|append\\|c\\(lose\\(file\\)?\\|reate\\)\\|edit\\|f\\(ind\\|orce\\|ree\\)"
+		"\\|insert\\|load\\|mattrib\\|p\\(a[ru]se\\|rint\\|urge\\)"
+		"\\|re\\(move\\|peat\\|place\\|set\\|sume\\)"
+		"\\|s\\(et\\(in\\|out\\)\\|how\\|ort\\|tore\\|ummary\\)\\|use\\)?"
+
 		"\\>")				    font-lock-keyword-face)
-;;	 (cons "\\<\\(\\(\\(then\\|else\\)[ \t]*\\)?do\\([ \t]*over\\)?\\|else\\)\\>"
-	 (cons "\\<\\(\\(then\\|else\\)[ \t]*\\)?\\(do\\([ \t]*over\\)?\\|else\\)\\>"
-						    font-lock-keyword-face)
+
+	
+
+;;	 (cons "\\<\\(\\(then\\|else\\)[ \t]*\\)?\\(do\\([ \t]*over\\)?\\|else\\)\\>"
+;;						    font-lock-keyword-face)
 
 	 ;; SAS statements that must be followed by a semi-colon
 	 (cons (concat
 		"\\(^[0-9]*\\|):\\|[;,]\\|then\\|else\\)[ \t]*"
-		"\\(cards4?\\|end\\|l\\(ist\\|ostcard\\)\\|page\\|return\\|stop\\)?"
+		"\\(cards4?\\|datalines\\|end\\|l\\(ostcard\\)\\|page\\|return\\|stop\\)?"
 		"[ \t]*;")			    font-lock-keyword-face)
 
 	 ;; SAS/GRAPH statements not handled above
@@ -457,7 +467,7 @@ number."
 		"\\(axis\\|legend\\|pattern\\|symbol\\)"
 		"\\([1-9][0-9]?\\)?\\>")	    font-lock-keyword-face)
 
-	 ;; SAS functions and SAS macro functions
+	 ;; SAS Datastep functions and SAS macro functions
 	 (cons "%[a-z_][a-z_0-9]*[ \t]*[(;]"
 						    font-lock-function-name-face)
 	 (cons "\\<call[ \t]+[a-z_][a-z_0-9]*[ \t]*("
@@ -490,16 +500,31 @@ number."
 		"\\|i\\(bessel\\|n\\(dexw\\|put[cn]\\)\\)\\|jbessel\\|put[cn]"
 		"\\|lowcase\\|quote\\|resolve\\|s\\(oundex\\|ysprod\\)\\|tr\\(anwrd\\|imn\\)"
 		
+;;;    ;; IML functions that are not also Datastep functions
+		"\\|a\\(ll\\|ny\\|pply\\|rmasim\\)\\|b\\(lock\\|ranks\\|tran\\)"
+		"\\|c\\(har\\|hoose\\|on\\(cat\\|tents\\|vexit\\|vmod\\)\\|ovlag\\|shape\\|usum\\|vexhull\\)"
+		"\\|d\\(atasets\\|esignf?\\|et\\|iag\\|o\\|uration\\)"
+		"\\|e\\(chelon\\|igv\\(al\\|ec\\)\\)\\|f\\(ft\\|orward\\)\\|ginv"
+		"\\|h\\(alf\\|ankel\\|dir\\|ermite\\|omogen\\)"
+		"\\|i\\(\\|fft\\|nsert\\|nv\\(updt\\)?\\)\\|j\\(\\|root\\)\\|loc\\|mad"
+		"\\|n\\(ame\\|col\\|leng\\|row\\|um\\)\\|o\\(pscal\\|rpol\\)"
+		"\\|p\\(olyroot\\|roduct\\|v\\)\\|r\\(anktie\\|ates\\|atio\\|emove\\|eturn\\|oot\\|owcatc?\\)"
+		"\\|s\\(etdif\\|hape\\|olve\\|plinev\\|pot\\|qrsym\\|ssq\\|torage\\|weep\\|ymsqr\\)"
+		"\\|t\\(\\|eigv\\(al\\|ec\\)\\|oeplitz\\|race\\|risolv\\|ype\\)"
+		"\\|uni\\(on\\|que\\)\\|v\\(alue\\|ecdiag\\)\\|x\\(mult\\|sect\\)\\|yield"
+		
 ;;;    ;; SCL functions that are known to work with SAS macro function %sysfunc
 
 		"\\|attr[cn]\\|c\\(exist\\|lose\\)\\|d\\(close\\|num\\|open\\|read\\)"
 		"\\|exist\\|f\\(close\\|etchobs\\|i\\(leexist\\|nfo\\)\\|open\\|put\\|write\\)"
 		"\\|get\\(option\\|var[cn]\\)\\|lib\\(name\\|ref\\)\\|op\\(en\\|t\\(getn\\|setn\\)\\)"
-		"\\|pathname\\|sysmsg\\|var\\(fmt\\|label\\|num\\|type\\)\\)"
+		"\\|pathname\\|sysmsg\\|var\\(fmt\\|l\\(abel\\|en\\)\\|n\\(ame\\|um\\)\\|type\\)\\)"
 		"[ \t]*(")			    font-lock-function-name-face)
 	 ))
 	"Font Lock regexs for SAS.")
-)); only if window-system
+)
+  (defvar SAS-mode-font-lock-keywords "") ;; empty if not window-system
+); only if window-system
 
 (defvar SAS-editing-alist
   '((sentence-end                 . ";[\t\n */]*")
